/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package beans;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import metier.Client;
import metier.CompteBancaire;
import metier.CompteCourant;
import metier.CompteEpargne;
import metier.OperationBancaire;

/**
 *
 * @author kaba
 */
@Stateless
public class GestionnaireClientBean implements GestionnaireClientLocal {

    @EJB
    private GestionnaireDeCompteBancaireLocal gestionnaireDeCompteBancaireBean;
    @PersistenceContext(unitName = "Benmosbah_Gaag_Kaba-ejbPU")
    private EntityManager em;

    public void persist(Object object) {
        em.persist(object);
    }

    @Override
    public void ajouterClient(Client clt) {
        em.persist(clt);
    }

    @Override
    public Client findClientById(Long id_client) {
        Query q = em.createQuery("SELECT clt FROM Client clt WHERE clt.id = :id_client");
        q.setParameter("id_client", id_client);
        return (Client) q.getSingleResult();
    }

    @Override
    public Client findClientByName(String nom_client) {
        Query q = em.createQuery("SELECT clt FROM Client clt WHERE clt.nom = :nom_client");
        q.setParameter("nom_client", nom_client);
        try {
            return (Client) q.getSingleResult();
        } catch (NoResultException e) {
            return null;
        } catch (NonUniqueResultException nu) {
            List<Client> c = (List<Client>) q.getResultList();

            return c.get(0);
        }
    }

    @Override
    public Collection<Client> findAllClient() {
        Query q = em.createQuery("SELECT clt FROM Client clt");
        return q.getResultList();
    }

    @Override
    public Client findClientByEmail(String email_client) {
        Query q = em.createQuery("SELECT clt FROM Client clt WHERE clt.email = :email_client");
        q.setParameter("email_client", email_client);
        try {
            return (Client) q.getSingleResult();
        } catch (NoResultException e) {
            return null;
        } catch (NonUniqueResultException nu) {
            List<Client> c = (List<Client>) q.getResultList();
            return c.get(0);
        }
    }

    @Override
    public Client findClientByTelephone(String telephone_client) {
        Query q = em.createQuery("SELECT clt FROM Client clt WHERE clt.telephone = :telephone_client");
        q.setParameter("telephone_client", telephone_client);
        return (Client) q.getSingleResult();
    }

    @Override
    public void creerClient(String nom, String prenom, String adresse, String email, String telephone, CompteBancaire cB) {
        Client c = findClientByName(nom);
        if (c != null) {
            c.addCompteBancaire(cB);
        } else {
            Client clt = new Client(nom, prenom, adresse, email, telephone);
            clt.addCompteBancaire(cB);
            em.persist(clt);
        }



    }

    @Override
    public void creerClient2(String nom, String prenom, String adresse, String email, String telephone) {
        Client clt = new Client(nom, prenom, adresse, email, telephone);
        em.persist(clt);
    }

    @Override
    public void creerClient(String nom, String prenom, String adresse, String email, String telephone, double solde, CompteBancaire.TypeCompte typeCompte) {

        CompteBancaire cB = new CompteBancaire(solde);
        if (typeCompte == CompteBancaire.TypeCompte.COMPTE_COURANT) {
            cB = new CompteCourant(solde);
        } else if (typeCompte == CompteBancaire.TypeCompte.COMPTE_EPARGNE) {
            cB = new CompteEpargne(solde);
        }

        OperationBancaire op = new OperationBancaire("Création du compte", solde);
        cB.addOperation(op);

        Client c = findClientByName(nom);
        if (c != null) {
            c.addCompteBancaire(cB);
        } else {
            Client clt = new Client(nom, prenom, adresse, email, telephone);
            clt.addCompteBancaire(cB);
            em.persist(clt);
        }



    }

    @Override
    public void creerClient(String nom, String prenom, String adresse, String email, String telephone, double solde) {

        CompteBancaire cB = new CompteBancaire(solde);
        OperationBancaire op = new OperationBancaire("Création du compte", solde);
        cB.addOperation(op);
        Client c = findClientByName(nom);
        if (c != null) {
            c.addCompteBancaire(cB);
        } else {
            Client clt = new Client(nom, prenom, adresse, email, telephone);
            clt.addCompteBancaire(cB);
            em.persist(clt);
        }



    }

    @Override
    public void creerClientTest() {
        creerClient("Baba", "Ali", "rue de Bagdad", "alibaba@gmail.com", "0604444", 10000, CompteBancaire.TypeCompte.COMPTE_COURANT);
        creerClient("Baba", "Ali", "rue de Bagdad", "alibaba@gmail.com", "0604444", 10000, CompteBancaire.TypeCompte.COMPTE_EPARGNE);
        creerClient("Potter", "Harry", "ecole de magie", "harrypotter@gmail.com", "0604004", 10000, CompteBancaire.TypeCompte.COMPTE_COURANT);
        creerClient("Potter", "Harry", "ecole de magie", "harrypotter@gmail.com", "0604004", 10000, CompteBancaire.TypeCompte.COMPTE_EPARGNE);
        creerClient("Botter", "Chat", "dans la foret", "lechatbotter@gmail.com", "555678", 10000, CompteBancaire.TypeCompte.COMPTE_COURANT);
        creerClient("Botter", "Chat", "dans la foret", "lechatbotter@gmail.com", "555678", 10000, CompteBancaire.TypeCompte.COMPTE_EPARGNE);
    }

    @Override
    public void supprimerClient(Long id) {

        Client c = findClientById(id);
        em.remove(c);
        /*  Query q = em.createQuery("delete from Client c where c.id=:id ").
        setParameter("id", id);
        q.executeUpdate();*/
    }

    @Override
    public void modifierClientById(Long id, String nom, String prenom, String adresse, String email, String telephone) {
        Query q = em.createQuery("update Client clt set clt.nom=:nom, clt.prenom=:prenom, clt.adresse=:adresse, clt.email=:email, clt.telephone=:telephone where clt.id=:id ").
                setParameter("id", id).
                setParameter("nom", nom).
                setParameter("prenom", prenom).
                setParameter("adresse", adresse).
                setParameter("email", email).
                setParameter("telephone", telephone);
        q.executeUpdate();
    }

    @Override
    public Collection<Long> findAllidCompteBancaireByIdClient(Long id_client) {
        try {
            //Récupération du client
            Client clt = findClientById(id_client);
            System.out.println("id client gestionnaire = " + id_client);
            //Récupération de tous les comptes de ce client
            Collection<CompteBancaire> listeComptesDuClient = clt.getListeComptes();
            System.out.println("liste des Comptes Du Client = " + listeComptesDuClient);

            Collection<Long> listeIDComptesDuClient = new ArrayList<Long>();
            //Collection<CompteBancaire> listeComptesDuClient2 = new ArrayList<CompteBancaire>();

            //Récupération de tous les id des comptes d'un client
            for (CompteBancaire compte : listeComptesDuClient) {
                Long id_compte = compte.getId();
                System.out.println("id compte gestionnaireclt = " + id_compte);
                listeIDComptesDuClient.add(id_compte);
                System.out.println(" liste id compteclient gestionnaireclt = " + listeIDComptesDuClient);

                CompteBancaire compteDuClient = gestionnaireDeCompteBancaireBean.findCompteBancaireById(id_compte);
                System.out.println("id compteclient gestionnaireclt = " + compteDuClient);


            }
            return listeIDComptesDuClient;
        } catch (NoResultException ex) {
            return null;
        }
    }

    @Override
    public void suprrimerComptesDunClient(Long id_client) {

        Collection<Long> listeIdCompteClient = findAllidCompteBancaireByIdClient(id_client);
        for (Long idcompte : listeIdCompteClient) {
            //CompteBancaire compteDuClient2 = gestionnaireDeCompteBancaireBean.findCompteBancaireById(idcompte);
            //Collection<OperationBancaire> listeOpDuCompte = compteDuClient2.getOperations();
            // em.remove(listeOpDuCompte);
            System.out.println("id supp  compteclient gestionnaireclt à supprimer = " + idcompte);
            gestionnaireDeCompteBancaireBean.supprimerCompteBancaire(idcompte);
            System.out.println("id compteclient supprimé gestionnaireclt = " + idcompte);
        }

    }
}
