/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package beans;

import java.util.ArrayList;
import java.util.Collection;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import metier.*;

/**
 *
 * @author kaba
 */
@Stateless
public class GestionnaireDeCompteBancaireBean implements GestionnaireDeCompteBancaireLocal {

    @PersistenceContext(unitName = "Benmosbah_Gaag_Kaba-ejbPU")
    private EntityManager em;

    public void persist(Object object) {
        em.persist(object);
    }

    @Override
    public void creerCompteBancaire(Client client, double solde) {

        OperationBancaire op = new OperationBancaire("Création du compte", solde);
        CompteBancaire cb = new CompteBancaire(client, solde);
        cb.addOperation(op);
        client.addCompteBancaire(cb);
        em.persist(client);
    }

    @Override
    public void creerCompteBancaire(Client client, double solde, CompteBancaire.TypeCompte typeCompte) {
        CompteBancaire cB = new CompteBancaire(client, solde);

        if (typeCompte == CompteBancaire.TypeCompte.COMPTE_COURANT) {
            cB = new CompteCourant(client, solde);
        } else if (typeCompte == CompteBancaire.TypeCompte.COMPTE_EPARGNE) {
            cB = new CompteEpargne(client, solde);
        }

        OperationBancaire op = new OperationBancaire("Création du compte", solde);
        cB.addOperation(op);
        client.addCompteBancaire(cB);
        em.persist(client);

    }

    @Override
    public void modifierCompteBancaire(Long id, double solde) {
        CompteBancaire cB = findCompteBancaireById(id);
        cB.setSolde(solde);
        OperationBancaire op = new OperationBancaire("Modification du solde", solde);
        cB.addOperation(op);
        em.persist(cB);

    }

    @Override
    public void afficherComptes() {
        Query query = em.createNamedQuery("findAll");
        Collection<CompteBancaire> result = query.getResultList();

        for (CompteBancaire c : result) {
            System.out.println(c);
        }

    }

    @Override
    public Collection<CompteBancaire> getAllComptes() {
        Query query = em.createNamedQuery("findAll");
        Collection<CompteBancaire> result = query.getResultList();

        return result;
    }

    @Override
    public void insererCompteTest() {

        creerCompteBancaire(new Client("Baba", "Ali", "rue de Bagdad", "alibaba@gmail.com", "0604444"), 155555, CompteBancaire.TypeCompte.COMPTE_COURANT);
        creerCompteBancaire(new Client("Potter", "Harry", "ecole de magie", "harrypotter@gmail.com", "0604004"), 155555, CompteBancaire.TypeCompte.COMPTE_EPARGNE);
        creerCompteBancaire(new Client("Botter", "Chat", "dans la foret", "lechatbotter@gmail.com", "555678"), 155555, CompteBancaire.TypeCompte.COMPTE_COURANT);
        creerCompteBancaire(new Client("John", "Joe", "a la fac", "john@gmail.com", "545578"), 155555, CompteBancaire.TypeCompte.COMPTE_EPARGNE);
    }

    @Override
    public CompteBancaire findCompteBancaireById(Long id) {
        try {
            return (CompteBancaire) em.createNamedQuery("findById").setParameter("id", id).getSingleResult();
        } catch (NoResultException ex) {
            return null;
        }
    }




    @Override
    public void ajouterMontant(Long id, double montant) throws AjoutMontantException {

        CompteBancaire cb = findCompteBancaireById(id);
        if (montant < 0) {
            throw new AjoutMontantException();
        } else {
            cb.ajouter(montant);
            OperationBancaire op = new OperationBancaire("Crédit", montant);
            cb.addOperation(op);
        }
    }

    @Override
    public void retirerMontant(Long id, double montant) throws RetraitMontantException {
        CompteBancaire cb = findCompteBancaireById(id);
        if (cb.getSolde() < montant) {
            throw new RetraitMontantException(cb.getSolde());

        } else {

            cb.retirer(montant);
            OperationBancaire op = new OperationBancaire("Débit", montant);
            cb.addOperation(op);
        }
    }

    @Override
    public void supprimerCompteBancaire(Long id) {
        CompteBancaire cb = findCompteBancaireById(id);
        em.remove(cb);
    }
}
